%% Membrane Data
L = 0.17; %Fiber Length [m]
d = 0.0007; %Fiber diameter [m]
Ri = d/2; %Fiber Radius [m]

vm = 1; %crossflow velocity [m/s]
vs = 1; %crossflow velocity [m/s]

%% Parameters
kB = physconst('Boltzmann'); %Boltzmann constant [J/K]
Rg = 8.314; %ideal gas constant [J/(mol*K)]
T = 293.15; %temperature [K]
F = 96500; %Faraday constant [C/mol]
n = 40; %x dimension (radial direction) grid points top layer
m = 40; %x dimension (radial direction) grid points bottom  layer
relax = 0.15; %relaxation coefficient
residual = 10^(-5); %residual DSPM&DE
maxIter = 10000; %maximum Number of Iterations DSPM^DE
eps_b = 80; %dielectric constant in bulk [-]
eta = 1.0016E-3; %water viscosity [Pa*s]
rho = 998.21; %density [kg/m^3]
Na = 6.02214076*10^23; %Avogadro constant [1/mol]
b = 0.3; %parameter in Davies model [-]
eps_0 = 8.854*10^(-12); %vacuum permittivity [C/(V*m)]
e0 = 1.602*10^(-19); %elementary charge [C]
cmass = 1; % concentration PEG [g/L]
MW = [62 106 200 400 600 1000 2000 3000]; % Molecular Weight PEG [g/mol]
Cf = cmass./MW*1000; % molar concentration PEG [mol/m^3]
D_inf = 1.25E-8*(T/298.15)*(0.89E-3/eta)*MW.^(-0.55); % Diffusion Coefficient PEG [m^2/s]
rS = kB*T./(6*pi*eta*D_inf); % Stokes Radii PEG [m]

%% Membrane Properties
%PAHPAA
rp1=2.8655637E-10;
dxe1=1.8712486E-07;%5.0342801E-07*1;
eps_p1=35;
Xd1=0.0043177;
%PAHPSS
rp2=4.8718118E-10;
dxe2=6.4963461E-07;
eps_p2=35;
Xd2=-0.04234;

Jtest=29.0176721721983/60/60/1000; % Flux

for j=1:1:size(Jtest,2)

% % %NaCl
% par = [eps_p1 Xd1 rp1 dxe1 eps_p2 Xd2 rp2 dxe2 [5 5] n m rho Ri vs ...
% eta [1.775e-9 1.166e-9] L Na e0 eps_0 kB T eps_b [-1 1] F Rg residual relax maxIter b];
% %NaSO4
% par = [eps_p1 Xd1 rp1 dxe1 eps_p2 Xd2 rp2 dxe2 [5 10] n m rho Ri vs ...
% eta [0.9305e-9 1.166e-9] L Na e0 eps_0 kB T eps_b [-2 1] F Rg residual relax maxIter b];
% %MgCl2
% par = [eps_p1 Xd1 rp1 dxe1 eps_p2 Xd2 rp2 dxe2 [5 10] n m rho Ri vs ...
% eta [0.6168e-9 1.775e-9] L Na e0 eps_0 kB T eps_b [2 -1] F Rg residual relax maxIter b];
%MgSO4
par = [eps_p1 Xd1 rp1 dxe1 eps_p2 Xd2 rp2 dxe2 [5 5] n m rho Ri vs ...
eta [0.6168e-9 0.9305e-9] L Na e0 eps_0 kB T eps_b [2 -2] F Rg residual relax maxIter b];

%overal retention
Ret(j)=SaltPlot(Jtest(j),par);
dP(j)=Jtest(j)/rp1^2*(8*eta*dxe1)+Jtest(j)/rp2^2*(8*eta*dxe2);
PWP(j)=Jtest(j)/dP(j)*1000*60*60*10^5;
end

Out1=['The Pure Water Permeability is: ',num2str(mean(PWP),10)];
disp(Out1);

RH=1./(PWP/1000/60/60/10^5)./eta;

Out2=['The Hydraulic Resistance is: ',num2str(mean(RH),10)];
disp(Out2);

% %Determine Corrected Thickness
% PWP1=rp1^2/(8*eta*dxe1);
% PWP2=rp2^2/(8*eta*dxe2);
% RH1=1./PWP1/eta;
% RH2=1./PWP2/eta;
% RH_C=RH1+RH2;
% PWP_C=1./RH/eta*1000*60*60*10^5;
% dx1_C=rp1^2/8*(1000*60*60*10^5/PWP_C/eta-8*dxe2/rp2^2);

function [rejection] = SaltPlot(x,par)
%parameters
eps_p1 = par(1); %effective dielectric constant top layer [m]
Xd1 = par(2); %effective charge density top layer [m]
rp1 = par(3); %effective pore radius top layer [m]
dxe1 = par(4); %effective membrane thickness top layer [m]
eps_p2 = par(5); %effective dielectric constant bottom layer [m]
Xd2 = par(6); %effective charge density bottom layer [m]
rp2 = par(7); %effective pore radius bottom layer [m]
dxe2 = par(8); %effective membrane thickness bottom layer [m]
Cf = par(9:10); %feed concentrations
n = par(11); %x dimension (radial direction) grid points top layer
m = par(12); %x dimension (radial direction) grid points bottom layer
rho = par(13); %density of water
Ri = par(14); %fiber diameter
v0 = par(15); %inlet flow velocity
eta = par(16); %viscosity of water
D_inf = par(17:18); %Diffusion coefficients
L = par(19); %fiber length
Na = par(20); %Avogadro constant
e0 = par(21); %elementary charge [C]
eps_0 = par(22); %vacuum permittivity [C/(V*m)]
kB = par(23); %Boltzmann constant [J/K]
T = par(24); %temperature [K]
eps_b = par(25); %dielectric constant in bulk [-]
zi = par(26:27); %ion charges
F = par(28); %Faraday constant [C/mol]
Rg = par(29); %ideal gas constant [J/(mol*K)]
residual = par(30); %residual DSPM&DE
relax = par(31); %relaxation coefficient
maxIter = par(32);
b = par(33); %parameter in Davies model [-]

Jv_1 = x;

%%parameter calculations
dx1=dxe1/(n-1); %grid node difference top layer [m]
dx2=dxe2/(n-1); %grid node difference bottom layer [m]
Re0=rho*2*Ri*v0/eta; %inlet Reynolds number [-]
Sc1=eta/rho/D_inf(1); %inlet Schmidt number [-]
Sc2=eta/rho/D_inf(2); %inlet Schmidt number [-]
Sh1=1.62*Re0^(1/3)*Sc1^(1/3)*(Ri*2/L)^(1/3)-1.2-0.28*Re0^(-1/3)*Sc1^(-1/3)*(Ri*2/L)^(-1/3); %inlet Sherwood number according to Newman [-]
Sh2=1.62*Re0^(1/3)*Sc2^(1/3)*(Ri*2/L)^(1/3)-1.2-0.28*Re0^(-1/3)*Sc2^(-1/3)*(Ri*2/L)^(-1/3); %inlet Sherwood number according to Newman [-]
kc1=Sh1/(2*Ri/D_inf(1)); 
kc2=Sh2/(2*Ri/D_inf(2));
phi1=Jv_1/kc1;
phi2=Jv_1/kc2;
suc1=phi1+(1+0.26*phi1.^(1.4)).^(-1.7);
suc2=phi2+(1+0.26*phi2.^(1.4)).^(-1.7);
kc1s=kc1*suc1;
kc2s=kc2*suc2;
A11 = (2*pi*Na)^(1/2)/log(10)*(e0^2/4/pi/eps_0/eps_p1/kB/T)^(3/2); %constant A1 top layer
An1 = (2*pi*Na)^(1/2)/log(10)*(e0^2/4/pi/eps_0/eps_p1/kB/T)^(3/2); %constant An top layer
A12 = (2*pi*Na)^(1/2)/log(10)*(e0^2/4/pi/eps_0/eps_p2/kB/T)^(3/2); %constant A1 bottom layer
An2 = (2*pi*Na)^(1/2)/log(10)*(e0^2/4/pi/eps_0/eps_p2/kB/T)^(3/2); %constant An bottom layer
Aw = (2*pi*Na)^(1/2)/log(10)*(e0^2/4/pi/eps_0/eps_b/kB/T)^(3/2); %constant Aw
Ap = (2*pi*Na)^(1/2)/log(10)*(e0^2/4/pi/eps_0/eps_b/kB/T)^(3/2); %constant Ap
rS1 = kB*T/(6*pi*eta*D_inf(1)); %Stokes radius 1
rS2 = kB*T/(6*pi*eta*D_inf(2)); %Stokes radius 2
lam11 = 0.0000001;%rS1/(rp1); %relative radii 1 top layer
lam21 = 0.0000001;%rS2/(rp1); %relative radii 2 top layer
lam12 = 0.0000001;%rS1/(rp2); %relative radii 1 bottom layer
lam22 = 0.0000001;%rS2/(rp2); %relative radii 2 bottom layer
%top layer
if lam11 <= 0.95
K1d1 = (1+9/8*lam11*log(lam11)-1.56034*lam11+0.528155*lam11^2+1.91521*lam11^3-2.81903*lam11^4+0.270788*lam11^5+1.10115*lam11^6-0.435933*lam11^7)/(1-lam11)^2; %d hindrance factor 1
else
K1d1 = 0.984*((1-lam11)/lam11)^(5/2); %d hindrance factor 1
end
if lam21 <= 0.95
K2d1 = (1+9/8*lam21*log(lam21)-1.56034*lam21+0.528155*lam21^2+1.91521*lam21^3-2.81903*lam21^4+0.270788*lam21^5+1.10115*lam21^6-0.435933*lam21^7)/(1-lam21)^2; %d hindrance factor 2
else
K2d1 = 0.984*((1-lam21)/lam21)^(5/2); %d hindrance factor 2
end
K1c1 = (1+3.867*lam11-1.907*lam11^2-0.834*lam11^3)/(1+1.867*lam11-0.741*lam11^2); %c hindrance factor 1
K2c1 = (1+3.867*lam21-1.907*lam21^2-0.834*lam21^3)/(1+1.867*lam21-0.741*lam21^2); %c hindrance factor 2
D1p1 = K1d1*D_inf(1); %pore diffusion 1
D2p1 = K2d1*D_inf(2); %pore diffusion 2
theta11 = (1-lam11)^2; %steric partitioning 1
theta21 = (1-lam21)^2; %steric partitioning 2
dW11 = zi(1)^2*e0^2/8/pi/eps_0/rS1*(1/eps_p1-1/eps_b); %solvation energy 1
dW21 = zi(2)^2*e0^2/8/pi/eps_0/rS2*(1/eps_p1-1/eps_b); %solvation energy 2
thetab11 = exp(-dW11/kB/T); %dielectric partitioning 1
thetab21 = exp(-dW21/kB/T); %dielectric partitioning 2
%bottom layer
if lam12 <= 0.95
K1d2 = (1+9/8*lam12*log(lam12)-1.56034*lam12+0.528155*lam12^2+1.91521*lam12^3-2.81903*lam12^4+0.270788*lam12^5+1.10115*lam12^6-0.435933*lam12^7)/(1-lam12)^2; %d hindrance factor 1
else
K1d2 = 0.984*((1-lam12)/lam12)^(5/2); %d hindrance factor 1
end
if lam22 <= 0.95
K2d2 = (1+9/8*lam22*log(lam22)-1.56034*lam22+0.528155*lam22^2+1.91521*lam22^3-2.81903*lam22^4+0.270788*lam22^5+1.10115*lam22^6-0.435933*lam22^7)/(1-lam22)^2; %d hindrance factor 2
else
K2d2 = 0.984*((1-lam22)/lam22)^(5/2); %d hindrance factor 2
end
K1c2 = (1+3.867*lam12-1.907*lam12^2-0.834*lam12^3)/(1+1.867*lam12-0.741*lam12^2); %c hindrance factor 1
K2c2 = (1+3.867*lam22-1.907*lam22^2-0.834*lam22^3)/(1+1.867*lam22-0.741*lam22^2); %c hindrance factor 2
D1p2 = K1d2*D_inf(1); %pore diffusion 1
D2p2 = K2d2*D_inf(2); %pore diffusion 2
theta12 = (1-lam12)^2; %steric partitioning 1
theta22 = (1-lam22)^2; %steric partitioning 2
dW12 = zi(1)^2*e0^2/8/pi/eps_0/rS1*(1/eps_p2-1/eps_b); %solvation energy 1
dW22 = zi(2)^2*e0^2/8/pi/eps_0/rS2*(1/eps_p2-1/eps_b); %solvation energy 2
thetab12 = exp(-dW12/kB/T); %dielectric partitioning 1
thetab22 = exp(-dW22/kB/T); %dielectric partitioning 2

%%initial guess
C1po=Cf(1);
C2po=Cf(2);
C1Vo=Cf(1);
C2Vo=Cf(2);
C1mo=Cf(1);
C2mo=Cf(2);
gamma11o1=1;
gamma21o1=1;
gamma1no1=1;
gamma2no1=1;
gamma11o2=1;
gamma21o2=1;
gamma1mo2=1;
gamma2mo2=1;
gamma1wo=1;
gamma2wo=1;
gamma1po=1;
gamma2po=1;
PsiPo=0;
c1o1=Cf(1);
c2o1=Cf(2);
c1o2=Cf(1);
c2o2=Cf(2);
Psio1=0;
Psio2=0;

%%paramters
p(1) = Jv_1;
p(2) = kc1s;
p(3) = kc2s;
p(4) = Cf(1);
p(5) = Cf(2);
p(6) = zi(1);
p(7) = zi(2);
p(8) = D_inf(1);
p(9) = D_inf(2);
p(10) = F;
p(11) = Rg;
p(12) = T;
p(13) = theta11;
p(14) = theta21;
p(15) = thetab11;
p(16) = thetab21;
p(17) = D1p1;
p(18) = D2p1;
p(19) = dx1;
p(20) = K1c1;
p(21) = K2c1;
p(22) = Xd1;
p(23) = gamma11o1;
p(24) = gamma21o1;
p(25) = gamma1no1;
p(26) = gamma2no1;
p(27) = theta12;
p(28) = theta22;
p(29) = thetab12;
p(30) = thetab22;
p(31) = D1p2;
p(32) = D2p2;
p(33) = dx2;
p(34) = K1c2;
p(35) = K2c2;
p(36) = Xd2;
p(37) = gamma11o2;
p(38) = gamma21o2;
p(39) = gamma1mo2;
p(40) = gamma2mo2;
p(41) = C1Vo;
p(42) = C2Vo;
p(43) = C1po;
p(44) = C2po;
p(45) = C1mo;
p(46) = C2mo;
p(47) = gamma1wo;
p(48) = gamma2wo;
p(49) = gamma1po;
p(50) = gamma2po;
p(51) = PsiPo;
p(51+1:51+n) = c1o1;
p(51+n+1:51+2*n) = c2o1;
p(51+2*n+1:51+3*n) = Psio1;
p(51+3*n+1:51+3*n+m) = c1o2;
p(51+3*n+m+1:51+3*n+2*m) = c2o2;
p(51+3*n+2*m+1:51+3*n+3*m) = Psio2;

res1=1;
counter=1;   

converge=true;

while res1>residual || res2>residual || res3>residual || res4>residual || res5>residual || res6>residual || res7>residual || res8>residual
res = Geraldes(p,n,m);
A = res(:,1:6+3*n+3*m);
B = res(:,6+3*n+3*m+1);
v=linsolve(A,B);

C1p = v(1);
C2p = v(2);
C1m = v(3);
C2m = v(4);
zeta = v(5);
PsiP = v(6);
c11 = v(6+1:6+n);
c21 = v(6+n+1:6+2*n);
Psi1 = v(6+2*n+1:6+3*n);
c12 = v(6+3*n+1:6+3*n+m);
c22 = v(6+3*n+m+1:6+3*n+2*m);
Psi2 = v(6+3*n+2*m+1:6+3*n+3*m);

%top layer
Psio1 = p(51+2*n+1:51+3*n)';
Psi1 = Psio1+relax*(Psi1-Psio1);
%bottom layer
Psio2 = p(51+3*n+2*m+1:51+3*n+3*m)';
Psi2 = Psio2+relax*(Psi2-Psio2);
PsiPo = p(51);
PsiP = PsiPo+relax*(PsiP-PsiPo);

%top layer
c1o1 = p(51+1:51+n)';
c2o1 = p(51+n+1:51+2*n)';
cr1 = min(abs(c1o1./(c11-c1o1)),1).*(c11-c1o1)./c1o1;
cr2 = min(abs(c2o1./(c21-c2o1)),1).*(c21-c2o1)./c2o1;
c11=c1o1.*(1+relax*cr1);
c21=c2o1.*(1+relax*cr2);
%bottom layer
c1o2 = p(51+3*n+1:51+3*n+m)';
c2o2 = p(51+3*n+m+1:51+3*n+2*m)';
cr1 = min(abs(c1o2./(c12-c1o2)),1).*(c12-c1o2)./c1o2;
cr2 = min(abs(c2o2./(c22-c2o2)),1).*(c22-c2o2)./c2o2;
c12=c1o2.*(1+relax*cr1);
c22=c2o2.*(1+relax*cr2);

C1po = p(43)';
C2po = p(44)';
crp1 = min(abs(C1po./(C1p-C1po)),1).*(C1p-C1po)./C1po;
crp2 = min(abs(C2po./(C2p-C2po)),1).*(C2p-C2po)./C2po;
C1p=C1po.*(1+relax*crp1);
C2p=C2po.*(1+relax*crp2);

C1mo = p(45)';
C2mo = p(46)';
crm1 = min(abs(C1mo./(C1m-C1mo)),1).*(C1m-C1mo)./C1mo;
crm2 = min(abs(C2mo./(C2m-C2mo)),1).*(C2m-C2mo)./C2mo;
C1m=C1mo.*(1+relax*crm1);
C2m=C2mo.*(1+relax*crm2);

I11=0.5*(zi(1)^2*c11(1)/1000+zi(2)^2*c21(1)/1000); %ionic strength 1 top layer
In1=0.5*(zi(1)^2*c11(n)/1000+zi(2)^2*c21(n)/1000); %ionic strength n top layer
I12=0.5*(zi(1)^2*c12(1)/1000+zi(2)^2*c22(1)/1000); %ionic strength 1 bottom layer
In2=0.5*(zi(1)^2*c12(m)/1000+zi(2)^2*c22(m)/1000); %ionic strength n bottom layer
Iw=0.5*(zi(1)^2*abs(C1m)/1000+zi(2)^2*abs(C2m)/1000); %ionic strength w
Ip=0.5*(zi(1)^2*abs(C1p)/1000+zi(2)^2*abs(C2p)/1000); %ionic strength p
gamma111=exp(-A11*zi(1)^2*(I11.^(1/2)./(1+I11.^(1/2))-b*I11)); %activity coefficient gamma11 top layer
gamma211=exp(-A11*zi(2)^2*(I11.^(1/2)./(1+I11.^(1/2))-b*I11)); %activity coefficient gamma11 top layer
gamma1n1=exp(-An1*zi(1)^2*(In1.^(1/2)./(1+In1.^(1/2))-b*In1)); %activity coefficient gamma11 top layer
gamma2n1=exp(-An1*zi(2)^2*(In1.^(1/2)./(1+In1.^(1/2))-b*In1)); %activity coefficient gamma11 top layer
gamma112=exp(-A12*zi(1)^2*(I12.^(1/2)./(1+I12.^(1/2))-b*I12)); %activity coefficient gamma11 bottom layer
gamma212=exp(-A12*zi(2)^2*(I12.^(1/2)./(1+I12.^(1/2))-b*I12)); %activity coefficient gamma11 bottom layer
gamma1m2=exp(-An2*zi(1)^2*(In2.^(1/2)./(1+In2.^(1/2))-b*In2)); %activity coefficient gamma11 bottom layer
gamma2m2=exp(-An2*zi(2)^2*(In2.^(1/2)./(1+In2.^(1/2))-b*In2)); %activity coefficient gamma11 bottom layer
gamma1w=exp(-Aw*zi(1)^2*(Iw.^(1/2)./(1+Iw.^(1/2))-b*Iw)); %activity coefficient gamma11 
gamma2w=exp(-Aw*zi(2)^2*(Iw.^(1/2)./(1+Iw.^(1/2))-b*Iw)); %activity coefficient gamma11 
gamma1p=exp(-Ap*zi(1)^2*(Ip.^(1/2)./(1+Ip.^(1/2))-b*Ip)); %activity coefficient gamma11 
gamma2p=exp(-Ap*zi(2)^2*(Ip.^(1/2)./(1+Ip.^(1/2))-b*Ip)); %activity coefficient gamma11 

p(23) = gamma111;
p(24) = gamma211;
p(25) = gamma1n1;
p(26) = gamma2n1;
p(37) = gamma112;
p(38) = gamma212;
p(39) = gamma1m2;
p(40) = gamma2m2;
p(43) = C1p;
p(44) = C2p;
p(45) = C1m;
p(46) = C2m;
p(47) = gamma1w;
p(48) = gamma2w;
p(49) = gamma1p;
p(50) = gamma2p;
p(51) = PsiP;
p(51+1:51+n) = c11;
p(51+n+1:51+2*n) = c21;
p(51+2*n+1:51+3*n) = Psi1;
p(51+3*n+1:51+3*n+m) = c12;
p(51+3*n+m+1:51+3*n+2*m) = c22;
p(51+3*n+2*m+1:51+3*n+3*m) = Psi2;

res1v(counter)=sum(abs(-D1p1*(c11(2:n)-c11(1:n-1))/dx1-0.5*zi(1)*(c11(2:n)+c11(1:n-1))*D1p1*F/Rg/T.*(Psi1(2:n)-Psi1(1:n-1))/dx1+0.5*K1c1*(c11(2:n)+c11(1:n-1))*Jv_1-Jv_1*C1p)/abs(Jv_1*C1p)); %NP top layer
res2v(counter)=sum(abs(-D2p1*(c21(2:n)-c21(1:n-1))/dx1-0.5*zi(2)*(c21(2:n)+c21(1:n-1))*D2p1*F/Rg/T.*(Psi1(2:n)-Psi1(1:n-1))/dx1+0.5*K2c1*(c21(2:n)+c21(1:n-1))*Jv_1-Jv_1*C2p)/abs(Jv_1*C2p)); %NP top layer
res3v(counter)=sum(abs(-D1p2*(c12(2:m)-c12(1:m-1))/dx2-0.5*zi(1)*(c12(2:m)+c12(1:m-1))*D1p2*F/Rg/T.*(Psi2(2:m)-Psi2(1:m-1))/dx2+0.5*K1c2*(c12(2:m)+c12(1:m-1))*Jv_1-Jv_1*C1p)/abs(Jv_1*C1p)); %NP bottom layer
res4v(counter)=sum(abs(-D2p2*(c22(2:m)-c22(1:m-1))/dx2-0.5*zi(2)*(c22(2:m)+c22(1:m-1))*D2p2*F/Rg/T.*(Psi2(2:m)-Psi2(1:m-1))/dx2+0.5*K2c2*(c22(2:m)+c22(1:m-1))*Jv_1-Jv_1*C2p)/abs(Jv_1*C2p)); %NP bottom layer
res5v(counter)=(sum(abs(zi(1)*c11+zi(2)*c21+Xd1))+sum(abs(zi(1)*c12+zi(2)*c22+Xd2))+abs(zi(1)*C1p+zi(2)*C2p))/((abs(Xd1)+abs(Xd2)+sum(abs(zi.*Cf)))); %electroneutrality
res6v(counter)=abs(c11(1)*gamma111/(C1m*gamma1w*theta11*exp(-(zi(1)*F/Rg/T)*(Psi1(1)))*exp(-dW11/kB/T))-1)+abs(c21(1)*gamma211/(C2m*gamma2w*theta21*exp(-(zi(2)*F/Rg/T)*(Psi1(1)))*exp(-dW21/kB/T))-1)+abs(c12(m)*gamma1m2/(C1p*gamma1p*theta12*exp(-(zi(1)*F/Rg/T)*(Psi2(m)-PsiP))*exp(-dW12/kB/T))-1)+abs(c22(m)*gamma2m2/(C2p*gamma2p*theta22*exp(-(zi(2)*F/Rg/T)*(Psi2(m)-PsiP))*exp(-dW22/kB/T))-1); %partition feed and permeate
res7v(counter)=abs(c12(1)*gamma112/(c11(n)*gamma1n1*theta12/theta11*exp(-dW12/kB/T)/exp(-dW11/kB/T)*exp(-(zi(1)*F/Rg/T)*(Psi2(1)-Psi1(n))))-1)+abs(c22(1)*gamma212/(c21(n)*gamma2n1*theta22/theta21*exp(-dW22/kB/T)/exp(-dW21/kB/T)*exp(-(zi(2)*F/Rg/T)*(Psi2(1)-Psi1(n))))-1); %partition interlayer
res8v(counter)=abs(-kc1s*(C1m-Cf(1))+Jv_1*C1m-zi(1)*C1m*D_inf(1)*F/Rg/T*zeta-Jv_1*C1p)/abs(Jv_1*C1p)+abs(-kc2s*(C2m-Cf(2))+Jv_1*C2m-zi(2)*C2m*D_inf(2)*F/Rg/T*zeta-Jv_1*C2p)/abs(Jv_1*C2p); %boundary layer

res1=res1v(counter);
res2=res2v(counter);
res3=res3v(counter);
res4=res4v(counter);
res5=res5v(counter);
res6=res6v(counter);
res7=res7v(counter);
res8=res8v(counter);

cnt=(1:1:counter);

counter=counter+1;
if counter >maxIter
    disp("iteration did not converge within "+maxIter+" iterations!");
converge=false;
    break;
end

end

C1p = v(1);
C2p = v(2);
C1m = v(3);
C2m = v(4);
zeta = v(5);
PsiP = v(6);
c11 = v(6+1:6+n);
c21 = v(6+n+1:6+2*n);
Psi1 = v(6+2*n+1:6+3*n);
c12 = v(6+3*n+1:6+3*n+m);
c22 = v(6+3*n+m+1:6+3*n+2*m);
Psi2 = v(6+3*n+2*m+1:6+3*n+3*m);

Jv_1=p(1);
kc1s=p(2);
kc2s=p(3);
Cf(1)=p(4);
Cf(2)=p(5);
zi(1)=p(6);
zi(2)=p(7);
D_inf(1)=p(8);
D_inf(2)=p(9);
F=p(10);
Rg=p(11);
T=p(12);
theta11=p(13);
theta21=p(14);
thetab11=p(15);
thetab21=p(16);
D1p1=p(17);
D2p1=p(18);
dx1=p(19);
K1c1=p(20);
K2c1=p(21);
Xd1=p(22);
gamma11o1=p(23);
gamma21o1=p(24);
gamma1no1=p(25);
gamma2no1=p(26);
theta12=p(27);
theta22=p(28);
thetab12=p(29);
thetab22=p(30);
D1p2=p(31);
D2p2=p(32);
dx2=p(33);
K1c2=p(34);
K2c2=p(35);
Xd2=p(36);
gamma11o2=p(37);
gamma21o2=p(38);
gamma1mo2=p(39);
gamma2mo2=p(40);
C1Vo=p(41);
C2Vo=p(42);
C1po=p(43);
C2po=p(44);
C1mo=p(45);
C2mo=p(46);
gamma1wo=p(47);
gamma2wo=p(48);
gamma1po=p(49);
gamma2po=p(50);
PsiPo=p(51);
c1o1=p(51+1:51+n);
c2o1=p(51+n+1:51+2*n);
Psio1=p(51+2*n+1:51+3*n);
c1o2=p(51+3*n+1:51+3*n+m);
c2o2=p(51+3*n+m+1:51+3*n+2*m);
Psio2=p(51+3*n+2*m+1:51+3*n+3*m);

ret = 1-C1p/Cf(1);

if converge==false
    ret=100;
end

rejection(1) = ret;

end

function res = Geraldes(p,n,m) %% 2 components
%%paramters
Jv = p(1);
kc1s = p(2);
kc2s = p(3);
C1b = p(4);
C2b = p(5);
z1 = p(6);
z2 = p(7);
D1inf = p(8);
D2inf = p(9);
F = p(10);
Rg = p(11);
T = p(12);
%layer 1
theta11 = p(13);
theta21 = p(14);
theta1b1 = p(15);
theta2b1 = p(16);
D1p1 = p(17);
D2p1 = p(18);
dx1 = p(19);
K1c1 = p(20);
K2c1 = p(21);
Xd1 = p(22);
gamma111 = p(23);
gamma211 = p(24);
gamma1n1 = p(25);
gamma2n1 = p(26);
%layer 2
theta12 = p(27);
theta22 = p(28);
theta1b2 = p(29);
theta2b2 = p(30);
D1p2 = p(31);
D2p2 = p(32);
dx2 = p(33);
K1c2 = p(34);
K2c2 = p(35);
Xd2 = p(36);
gamma112 = p(37);
gamma212 = p(38);
gamma1m2 = p(39);
gamma2m2 = p(40);
%virtual interlayer concentration
C1Vo = p(41);
C2Vo = p(42);
C1po = p(43);
C2po = p(44);
C1mo = p(45);
C2mo = p(46);
gamma1w = p(47);
gamma2w = p(48);
gamma1p = p(49);
gamma2p = p(50);
PsiPo = p(51);
%layer 1
c1o1 = p(51+1:51+n);
c2o1 = p(51+n+1:51+2*n);
Psio1 = p(51+2*n+1:51+3*n);
%layer 2
c1o2 = p(51+3*n+1:51+3*n+m);
c2o2 = p(51+3*n+m+1:51+3*n+2*m);
Psio2 = p(51+3*n+2*m+1:51+3*n+3*m);

%%variable - actually not used here, just for orientation
% C1p = v(1);
% C2p = v(2);
% C1m = v(3);
% C2m = v(4);
% zeta = v(5);
% PsiP = v(6);
% c11 = v(6+1:6+n);
% c21 = v(6+n+1:6+2*n);
% Psi1 = v(6+2*n+1:6+3*n);
% c12 = v(6+3*n+1:6+3*n+m);
% c22 = v(6+3*n+m+1:6+3*n+2*m);
% Psi2 = v(6+3*n+2*m+1:6+3*n+3*m);

%boundary layer
A1 = zeros(2,6+3*n+3*m);
A1(1,1) = Jv;
A1(2,2) = Jv;
A1(1,3) = kc1s-Jv;
A1(2,4) = kc2s-Jv;
A1(1,5) = z1*C1mo*D1inf*F/Rg/T;
A1(2,5) = z2*C2mo*D2inf*F/Rg/T;
B1 = [kc1s*C1b;
      kc2s*C2b];
  
%electroneutrality feed
A2 = zeros(1,6+3*n+3*m);
A2(1,3) = z1;
A2(1,4) = z2;
B2 = 0;

%feed exclusion
A3 = zeros(2,6+3*n+3*m);
A3(1,6+1) = gamma111;
A3(2,6+n+1) = gamma211;
A3(1,6+2*n+1) = gamma1w*C1mo*z1*F/Rg/T*theta11*theta1b1*exp(-z1*F/Rg/T*Psio1(1));
A3(2,6+2*n+1) = gamma2w*C2mo*z2*F/Rg/T*theta21*theta2b1*exp(-z2*F/Rg/T*Psio1(1));
B3 = [gamma1w*C1mo*theta11*theta1b1*exp(-z1*F/Rg/T*Psio1(1))*(1+z1*F/Rg/T*Psio1(1));
      gamma2w*C2mo*theta21*theta2b1*exp(-z2*F/Rg/T*Psio1(1))*(1+z2*F/Rg/T*Psio1(1))];

%Nernst-Planck top layer
A4 = zeros((n-1)*2,6+3*n+3*m);
for i=1:n-1
A4(i,6+i) = D1p1/dx1+0.5*K1c1*Jv-0.5*z1*D1p1*F/Rg/T*(Psio1(i+1)-Psio1(i))/dx1;
end
for i=1:n-1
A4((n-1)+i,6+n+i) = D2p1/dx1+0.5*K2c1*Jv-0.5*z2*D2p1*F/Rg/T*(Psio1(i+1)-Psio1(i))/dx1;
end

for i=1:n-1
A4(i,6+i+1) = -D1p1/dx1+0.5*K1c1*Jv-0.5*z1*D1p1*F/Rg/T*(Psio1(i+1)-Psio1(i))/dx1;
end
for i=1:n-1
A4((n-1)+i,6+n+i+1) = -D2p1/dx1+0.5*K2c1*Jv-0.5*z2*D2p1*F/Rg/T*(Psio1(i+1)-Psio1(i))/dx1;
end

for i=1:n-1
A4(i,6+2*n+i) = 0.5*z1*D1p1*F/Rg/T*(c1o1(i+1)+c1o1(i))/dx1;
end
for i=1:n-1
A4((n-1)+i,6+2*n+i) = 0.5*z2*D2p1*F/Rg/T*(c2o1(i+1)+c2o1(i))/dx1;
end

for i=1:n-1
A4(i,6+2*n+i+1) = -0.5*z1*D1p1*F/Rg/T*(c1o1(i+1)+c1o1(i))/dx1;
end
for i=1:n-1
A4((n-1)+i,6+2*n+i+1) = -0.5*z2*D2p1*F/Rg/T*(c2o1(i+1)+c2o1(i))/dx1;
end

for i=1:n-1
A4(i,1) = -Jv;
end
for i=1:n-1
A4((n-1)+i,2) = -Jv;
end

B4 = zeros((n-1)*2,1);
for i=1:n-1
B4(i) = -0.5*z1*D1p1*F/Rg/T*(c1o1(i)+c1o1(i+1))*(Psio1(i+1)-Psio1(i))/dx1;
end
for i=1:n-1
B4((n-1)+i) = -0.5*z2*D2p1*F/Rg/T*(c2o1(i)+c2o1(i+1))*(Psio1(i+1)-Psio1(i))/dx1;
end

%interface bilayer
Theta1=theta12*theta1b2/(theta11*theta1b1);
Theta2=theta22*theta2b2/(theta21*theta2b1);

A5 = zeros(2,6+3*n+3*m);

A5(1,6+3*n+1) = gamma112;
A5(2,6+3*n+m+1) = gamma212;

A5(1,6+n) = -gamma1n1*Theta1*exp(-z1*F/Rg/T*(Psio2(1)-Psio1(n)));
A5(2,6+2*n) = -gamma2n1*Theta2*exp(-z2*F/Rg/T*(Psio2(1)-Psio1(n)));

A5(1,6+3*n+2*m+1) = z1*F/Rg/T*gamma1n1*c1o1(n)*Theta1*exp(-z1*F/Rg/T*(Psio2(1)-Psio1(n)));
A5(2,6+3*n+2*m+1) = z2*F/Rg/T*gamma2n1*c2o1(n)*Theta2*exp(-z2*F/Rg/T*(Psio2(1)-Psio1(n)));

A5(1,6+3*n) = -z1*F/Rg/T*gamma1n1*c1o1(n)*Theta1*exp(-z1*F/Rg/T*(Psio2(1)-Psio1(n)));
A5(2,6+3*n) = -z2*F/Rg/T*gamma2n1*c2o1(n)*Theta2*exp(-z2*F/Rg/T*(Psio2(1)-Psio1(n)));

B5 = [(Psio2(1)-Psio1(n))*z1*F/Rg/T*gamma1n1*c1o1(n)*Theta1*exp(-z1*F/Rg/T*(Psio2(1)-Psio1(n)));
      (Psio2(1)-Psio1(n))*z2*F/Rg/T*gamma2n1*c2o1(n)*Theta2*exp(-z2*F/Rg/T*(Psio2(1)-Psio1(n)))];

%Nernst-Planck bottom layer
A6 = zeros((m-1)*2,6+3*n+3*m);
for i=1:m-1
A6(i,6+3*n+i) = D1p2/dx2+0.5*K1c2*Jv-0.5*z1*D1p2*F/Rg/T*(Psio2(i+1)-Psio2(i))/dx2;
end
for i=1:m-1
A6((m-1)+i,6+3*n+m+i) = D2p2/dx2+0.5*K2c2*Jv-0.5*z2*D2p2*F/Rg/T*(Psio2(i+1)-Psio2(i))/dx2;
end

for i=1:m-1
A6(i,6+3*n+i+1) = -D1p2/dx2+0.5*K1c2*Jv-0.5*z1*D1p2*F/Rg/T*(Psio2(i+1)-Psio2(i))/dx2;
end
for i=1:m-1
A6((m-1)+i,6+3*n+m+i+1) = -D2p2/dx2+0.5*K2c2*Jv-0.5*z2*D2p2*F/Rg/T*(Psio2(i+1)-Psio2(i))/dx2;
end

for i=1:m-1
A6(i,6+3*n+2*m+i) = 0.5*z1*D1p2*F/Rg/T*(c1o2(i+1)+c1o2(i))/dx2;
end
for i=1:m-1
A6((m-1)+i,6+3*n+2*m+i) = 0.5*z2*D2p2*F/Rg/T*(c2o2(i+1)+c2o2(i))/dx2;
end

for i=1:m-1
A6(i,6+3*n+2*m+i+1) = -0.5*z1*D1p2*F/Rg/T*(c1o2(i+1)+c1o2(i))/dx2;
end
for i=1:m-1
A6((m-1)+i,6+3*n+2*m+i+1) = -0.5*z2*D2p2*F/Rg/T*(c2o2(i+1)+c2o2(i))/dx2;
end

for i=1:m-1
A6(i,1) = -Jv;
end
for i=1:m-1
A6((n-1)+i,2) = -Jv;
end

B6 = zeros((m-1)*2,1);
for i=1:m-1
B6(i) = -0.5*z1*D1p2*F/Rg/T*(c1o2(i)+c1o2(i+1))*(Psio2(i+1)-Psio2(i))/dx2;
end
for i=1:m-1
B6((m-1)+i) = -0.5*z2*D2p2*F/Rg/T*(c2o2(i)+c2o2(i+1))*(Psio2(i+1)-Psio2(i))/dx2;
end

%permeate exclusion
A7 = zeros(2,6+3*n+3*m);

A7(1,6+3*n+m) = gamma1m2;
A7(2,6+3*n+2*m) = gamma2m2;

A7(1,1) = -theta12*theta1b2*gamma1p*exp(z1*F/Rg/T*(PsiPo-Psio2(m)));
A7(2,2) = -theta22*theta2b2*gamma2p*exp(z2*F/Rg/T*(PsiPo-Psio2(m)));

A7(1,6+3*n+3*m) = gamma1p*C1po*z1*F/Rg/T*theta12*theta1b2*exp(z1*F/Rg/T*(PsiPo-Psio2(m)));
A7(2,6+3*n+3*m) = gamma2p*C2po*z2*F/Rg/T*theta22*theta2b2*exp(z2*F/Rg/T*(PsiPo-Psio2(m)));

A7(1,6) = -gamma1p*C1po*z1*F/Rg/T*theta12*theta1b2*exp(z1*F/Rg/T*(PsiPo-Psio2(m)));
A7(2,6) = -gamma2p*C2po*z2*F/Rg/T*theta22*theta2b2*exp(z2*F/Rg/T*(PsiPo-Psio2(m)));

B7 = [-gamma1p*C1po*theta12*theta1b2*exp(z1*F/Rg/T*(PsiPo-Psio2(m)))*(z1*F/Rg/T*(PsiPo-Psio2(m)));
      -gamma2p*C2po*theta22*theta2b2*exp(z2*F/Rg/T*(PsiPo-Psio2(m)))*(z2*F/Rg/T*(PsiPo-Psio2(m)))];
  
%electroneutrality permeate
A8 = zeros(1,6+3*n+3*m);
A8(1,1) = z1;
A8(1,2) = z2;
B8 = 0;

%electroneutrality membrane top layer
A9 = zeros(n,6+3*n+3*m);
for i=1:n
A9(i,6+i) = z1;
A9(i,6+n+i) = z2;
end
B9 = ones(n,1)*-Xd1;

%electroneutrality membrane bottom layer
A10 = zeros(n,6+3*n+3*m);
for i=1:m
A10(i,6+3*n+i) = z1;
A10(i,6+3*n+m+i) = z2;
end
B10 = ones(m,1)*-Xd2;

At = [A1;A2;A3;A4;A5;A6;A7;A8;A9;A10];
Bt = [B1;B2;B3;B4;B5;B6;B7;B8;B9;B10];

res = [At,Bt];
end
