%% Membrane Data
L = 0.17; %Fiber Length [m]
d = 0.0007; %Fiber diameter [m]
Ri = d/2; %Fiber Radius [m]

vm = 1; %crossflow velocity [m/s]
vs = 1; %crossflow velocity [m/s]

%% Parameters
kB = physconst('Boltzmann'); %Boltzmann constant [J/K]
Rg = 8.314; %ideal gas constant [J/(mol*K)]
T = 293.15; %temperature [K]
F = 96500; %Faraday constant [C/mol]
n = 40; %x dimension (radial direction) grid points
relax = 0.15; %relaxation coefficient
residual = 10^(-5); %residual DSPM&DE
maxIter = 1000; %maximum Number of Iterations DSPM^DE
eps_b = 80; %dielectric constant in bulk [-]
eta = 1.0016E-3; %water viscosity [Pa*s]
rho = 998.21; %density [kg/m^3]
Na = 6.02214076*10^23; %Avogadro constant [1/mol]
b = 0.3; %parameter in Davies model [-]
eps_0 = 8.854*10^(-12); %vacuum permittivity [C/(V*m)]
e0 = 1.602*10^(-19); %elementary charge [C]
cmass = 1; % concentration PEG [g/L]
MW = (10:10:80);%[62 106 200 400 600 1000 2000 3000]; % Molecular Weight PEG [g/mol] %
Cf = cmass./MW*1000; % molar concentration PEG [mol/m^3]
D_inf = 1.25E-8*(T/298.15)*(0.89E-3/eta)*MW.^(-0.55); % Diffusion Coefficient PEG [m^2/s]
rS = kB*T./(6*pi*eta*D_inf); % Stokes Radii PEG [m]

%% Pore size distribution
%PAHPAA
rmean1 = 2.7087788E-10;
std1 = 4.353327E-11;
dxen1 = 1.506E-25*8;%4.80E-25;
%PAHPSS
rmean2 = 3.7844316E-10;
std2 = 1.1101778E-10;
dxen2 = 2.9609848E-25;

deltar = 1E-11; %discretization size [m]
maxr = 10E-9; %upper limit [m]
bND = log(1+(std1/rmean1)^2);
rmean = rmean1;
rDist=deltar:deltar:maxr; %radius range [m]
for i=1:1:maxr/deltar %discrete function
Fr(i) = integral(@(r) 1./(r*(2*pi()*bND).^0.5).*exp(-(log(r./rmean)+bND/2).^2/2/bND),deltar*(i)-deltar/2,deltar*(i)+deltar/2); %relative ammount
end
Jvr=Fr.*rDist.^4; %flux as function of radius *dPeff /dxen/8/eta*pi()
RelFr=Jvr./sum(Jvr);
%truncation
maxf=max(RelFr);
minf=maxf*0.05; %5% of maximum value cut off
minc=0;
maxc=0.1;
for i=1:1:size(rDist,2)
if RelFr(i)>minf && minc<maxc
    minc=i;
end
if RelFr(i)<minf && minc>maxc
    maxc=i;
    break;
end
end
if maxc==0.1
   maxc=length(RelFr); 
end
if minc==0
   minc=1; 
end
rDist1=rDist(minc:maxc);
Fr=Fr(minc:maxc);
scale=sum(Fr); %normalize distribution to sumfR=1
Fr1=Fr./scale; 
Jvr=Fr1.*rDist1.^4; %flux as function of radius *dPeff /dxen/8/eta*pi()
RelFr1=Jvr./sum(Jvr);
K11=sum(pi()*rDist1.^4.*Fr1)/dxen1;
PWP1=K11*1000*60*60*10^5/8/eta;

bND = log(1+(std2/rmean2)^2);
rmean = rmean2;
rDist=deltar:deltar:maxr; %radius range [m]
for i=1:1:maxr/deltar %discrete function
Fr(i) = integral(@(r) 1./(r*(2*pi()*bND).^0.5).*exp(-(log(r./rmean)+bND/2).^2/2/bND),deltar*(i)-deltar/2,deltar*(i)+deltar/2); %relative ammount
end
Jvr=Fr.*rDist.^4; %flux as function of radius *dPeff /dxen/8/eta*pi()
RelFr=Jvr./sum(Jvr);
%truncation
maxf=max(RelFr);
minf=maxf*0.05; %5% of maximum value cut off
minc=0;
maxc=0.1;
for i=1:1:size(rDist,2)
if RelFr(i)>minf && minc<maxc
    minc=i;
end
if RelFr(i)<minf && minc>maxc
    maxc=i;
    break;
end
end
if maxc==0.1
   maxc=length(RelFr); 
end
if minc==0
   minc=1; 
end
rDist2=rDist(minc:maxc);
Fr=Fr(minc:maxc);
scale=sum(Fr); %normalize distribution to sumfR=1
Fr2=Fr./scale; 
Jvr=Fr2.*rDist2.^4; %flux as function of radius *dPeff /dxen/8/eta*pi()
RelFr2=Jvr./sum(Jvr);
K21=sum(pi()*rDist2.^4.*Fr2)/dxen2;
PWP2=K21*1000*60*60*10^5/8/eta;

Jtest=1.0065E-06;

for j=1:1:size(Jtest,2)
clear Ret

%flux as function of r
K11=sum(pi()*rDist1.^4.*Fr1)/(dxen1);
dPeff=Jtest(j)*8*eta/K11;
Jvr=Fr1.*pi().*rDist1.^4.*dPeff/8/eta/(dxen1);

par=[K11 dxen1 D_inf rS eta];
for i=1:1:size(Fr1,2)
Ret(:,i,j)=PoreSizeDistributionIntrinsicPlot([Jtest(j) rDist1(i) Fr1(i)],par);
end
%overal retention
Reto1(1,j)=sum((Ret(1,:,j)).*Jvr)/Jtest(j);
Reto1(2,j)=sum((Ret(2,:,j)).*Jvr)/Jtest(j);
Reto1(3,j)=sum((Ret(3,:,j)).*Jvr)/Jtest(j);
Reto1(4,j)=sum((Ret(4,:,j)).*Jvr)/Jtest(j);
Reto1(5,j)=sum((Ret(5,:,j)).*Jvr)/Jtest(j);
Reto1(6,j)=sum((Ret(6,:,j)).*Jvr)/Jtest(j);
Reto1(7,j)=sum((Ret(7,:,j)).*Jvr)/Jtest(j);
Reto1(8,j)=sum((Ret(8,:,j)).*Jvr)/Jtest(j);

dP(j)=dPeff;

clear Ret

%flux as function of r
K21=sum(pi()*rDist2.^4.*Fr2)/(dxen2);
dPeff=Jtest(j)*8*eta/K21;
Jvr=Fr2.*pi().*rDist2.^4.*dPeff/8/eta/(dxen2);

par=[K21 dxen2 D_inf rS eta];
for i=1:1:size(Fr2,2)
Ret(:,i,j)=PoreSizeDistributionIntrinsicPlot([Jtest(j) rDist2(i) Fr2(i)],par);
end
%overal retention
Reto2(1,j)=sum((Ret(1,:,j)).*Jvr)/Jtest(j);
Reto2(2,j)=sum((Ret(2,:,j)).*Jvr)/Jtest(j);
Reto2(3,j)=sum((Ret(3,:,j)).*Jvr)/Jtest(j);
Reto2(4,j)=sum((Ret(4,:,j)).*Jvr)/Jtest(j);
Reto2(5,j)=sum((Ret(5,:,j)).*Jvr)/Jtest(j);
Reto2(6,j)=sum((Ret(6,:,j)).*Jvr)/Jtest(j);
Reto2(7,j)=sum((Ret(7,:,j)).*Jvr)/Jtest(j);
Reto2(8,j)=sum((Ret(8,:,j)).*Jvr)/Jtest(j);

Ret_Int(:,j)=1-(1-Reto1(:,j)).*(1-Reto2(:,j));

Re0=rho*2*Ri*vm/eta; %inlet Reynolds number [-]
Sc=eta/rho./D_inf; %inlet Schmidt number [-]
Sh=1.62*Re0^(1/3)*Sc.^(1/3)*(Ri*2./L)^(1/3)-1.2-0.28*Re0^(-1/3)*Sc.^(-1/3)*(Ri*2./L)^(-1/3); %inlet Sherwood number according to Newman [-]
kc=Sh./(2*Ri./D_inf); 
phi=Jtest(j)./kc;
suc=phi+(1+0.26*phi.^(1.4)).^(-1.7);
kcs=kc.*suc;

Ret_CP(:,j)=1-((1-Ret_Int(:,j)).*kcs')./(kcs'-Ret_Int(:,j).*Jtest(j));
dP(j)=dP(j)+dPeff;
PWP(j)=Jtest(j)/dP(j)*1000*60*60*10^5;
end

Out1=['The Pure Water Permeability is: ',num2str(mean(PWP),10)];
disp(Out1);

function [retention] = PoreSizeDistributionIntrinsicPlot(x,param)
%parameters
K1 = param(1);
dxen=param(2);
D_inf = param(3:10);
rS1 = param(11);
rS2 = param(12);
rS3 = param(13);
rS4 = param(14);
rS5 = param(15);
rS6 = param(16);
rS7 = param(17);
rS8 = param(18);
eta = param(19);

%variables
Jv=x(1); %flux
rp=x(2); %pore size
Fr=x(3); %relative ammount

dxe=dxen/Fr/pi()./(rp.^2);
dPeff=Jv*8*eta/K1;
Jv=rp^2.*dPeff/8/eta/dxe;

%%EG
lam1=rS1/rp; %relative radii
if lam1>1
lam1=1;
end
if lam1 <= 0.95
K1d = (1+9/8*lam1*log(lam1)-1.56034*lam1+0.528155*lam1^2+1.91521*lam1^3-2.81903*lam1^4+0.270788*lam1^5+1.10115*lam1^6-0.435933*lam1^7)/(1-lam1)^2; %d hindrance factor
else
K1d = 0.984*((1-lam1)/lam1)^(5/2); %d hindrance factor
end
K1c = (1+3.867*lam1-1.907*lam1^2-0.834*lam1^3)/(1+1.867*lam1-0.741*lam1^2); %c hindrance factor
D1p = K1d*D_inf(1); %pore diffusion
theta1 = (1-lam1)^2; %steric partitioning

Pem=K1c*Jv*dxe/D1p;
ret_62_int=1-K1c*theta1/((K1c*theta1-1)*exp(-Pem)+1);
ret_62=ret_62_int;

%%DEG
lam2=rS2/rp; 
if lam2>1
lam2=1;
end
if lam2 <= 0.95
K2d = (1+9/8*lam2*log(lam2)-1.56034*lam2+0.528155*lam2^2+1.91521*lam2^3-2.81903*lam2^4+0.270788*lam2^5+1.10115*lam2^6-0.435933*lam2^7)/(1-lam2)^2; 
else
K2d = 0.984*((1-lam2)/lam2)^(5/2); 
end
K2c = (1+3.867*lam2-1.907*lam2^2-0.834*lam2^3)/(1+1.867*lam2-0.741*lam2^2); 
D2p = K2d*D_inf(2); 
theta2 = (1-lam2)^2; 

Pem=K2c*Jv*dxe/D2p;
ret_106_int=1-K2c*theta2/((K2c*theta2-1)*exp(-Pem)+1);
ret_106=ret_106_int;

%%PEG200
lam3=rS3/rp; 
if lam3>1
lam3=1;
end
if lam3 <= 0.95
K3d = (1+9/8*lam3*log(lam3)-1.56034*lam3+0.528155*lam3^2+1.91521*lam3^3-2.81903*lam3^4+0.270788*lam3^5+1.10115*lam3^6-0.435933*lam3^7)/(1-lam3)^2; 
else
K3d = 0.984*((1-lam3)/lam3)^(5/2); 
end
K3c = (1+3.867*lam3-1.907*lam3^2-0.834*lam3^3)/(1+1.867*lam3-0.741*lam3^2);
D3p = K3d*D_inf(3);
theta3 = (1-lam3)^2; 

Pem=K3c*Jv*dxe/D3p;
ret_200_int=1-K3c*theta3/((K3c*theta3-1)*exp(-Pem)+1);
ret_200=ret_200_int;

%%PEG400
lam4=rS4/rp; 
if lam4>1
lam4=1;
end
if lam4 <= 0.95
K4d = (1+9/8*lam4*log(lam4)-1.56034*lam4+0.528155*lam4^2+1.91521*lam4^3-2.81903*lam4^4+0.270788*lam4^5+1.10115*lam4^6-0.435933*lam4^7)/(1-lam4)^2; 
else
K4d = 0.984*((1-lam4)/lam4)^(5/2); 
end
K4c = (1+3.867*lam4-1.907*lam4^2-0.834*lam4^3)/(1+1.867*lam4-0.741*lam4^2); 
D4p = K4d*D_inf(4); 
theta4 = (1-lam4)^2; 

Pem=K4c*Jv*dxe/D4p;
ret_400_int=1-K4c*theta4/((K4c*theta4-1)*exp(-Pem)+1);
ret_400=ret_400_int;

%%PEG600
lam5=rS5/rp; 
if lam5>1
lam5=1;
end
if lam5 <= 0.95
K5d = (1+9/8*lam5*log(lam5)-1.56034*lam5+0.528155*lam5^2+1.91521*lam5^3-2.81903*lam5^4+0.270788*lam5^5+1.10115*lam5^6-0.435933*lam5^7)/(1-lam5)^2; 
else
K5d = 0.984*((1-lam5)/lam5)^(5/2); 
end
K5c = (1+3.867*lam5-1.907*lam5^2-0.834*lam5^3)/(1+1.867*lam5-0.741*lam5^2); 
D5p = K5d*D_inf(5); 
theta5 = (1-lam5)^2; 

Pem=K5c*Jv*dxe/D5p;
ret_600_int=1-K5c*theta5/((K5c*theta5-1)*exp(-Pem)+1);
ret_600=ret_600_int;

%%PEG1000
lam6=rS6/rp;
if lam6>1
lam6=1;
end
if lam6 <= 0.95
K6d = (1+9/8*lam6*log(lam6)-1.56034*lam6+0.528155*lam6^2+1.91521*lam6^3-2.81903*lam6^4+0.270788*lam6^5+1.10115*lam6^6-0.435933*lam6^7)/(1-lam6)^2; %d hindrance factor 1
else
K6d = 0.984*((1-lam6)/lam6)^(5/2);
end
K6c = (1+3.867*lam6-1.907*lam6^2-0.834*lam6^3)/(1+1.867*lam6-0.741*lam6^2);
D6p = K6d*D_inf(6);
theta6 = (1-lam6)^2;

Pem=K6c*Jv*dxe/D6p;
ret_1000_int=1-K6c*theta6/((K6c*theta6-1)*exp(-Pem)+1);
ret_1000=ret_1000_int;

%%PEG2000
lam7=rS7/rp;
if lam7>1
lam7=1;
end
if lam7 <= 0.95
K7d = (1+9/8*lam7*log(lam7)-1.56034*lam7+0.528155*lam7^2+1.91521*lam7^3-2.81903*lam7^4+0.270788*lam7^5+1.10115*lam7^6-0.435933*lam7^7)/(1-lam7)^2; 
else
K7d = 0.984*((1-lam7)/lam7)^(5/2); 
end
K7c = (1+3.867*lam7-1.907*lam7^2-0.834*lam7^3)/(1+1.867*lam7-0.741*lam7^2); 
D7p = K7d*D_inf(7); 
theta7 = (1-lam7)^2;

Pem=K7c*Jv*dxe/D7p;
ret_2000_int=1-K7c*theta7/((K7c*theta7-1)*exp(-Pem)+1);
ret_2000=ret_2000_int;

%%PEG3000
lam8=rS8/rp;
if lam8>1
lam8=1;
end
if lam8 <= 0.95
K8d = (1+9/8*lam8*log(lam8)-1.56034*lam8+0.528155*lam8^2+1.91521*lam8^3-2.81903*lam8^4+0.270788*lam8^5+1.10115*lam8^6-0.435933*lam8^7)/(1-lam8)^2;
else
K8d = 0.984*((1-lam8)/lam8)^(5/2); 
end
K8c = (1+3.867*lam8-1.907*lam8^2-0.834*lam8^3)/(1+1.867*lam8-0.741*lam8^2);
D8p = K8d*D_inf(8); 
theta8 = (1-lam8)^2;

Pem=K8c*Jv*dxe/D8p;
ret_3000_int=1-K8c*theta8/((K8c*theta8-1)*exp(-Pem)+1);
ret_3000=ret_3000_int;

retention = [ret_62;ret_106;ret_200;ret_400;ret_600;ret_1000;ret_2000;ret_3000];
end


